using System;
using System.Web.Services.Protocols;

using VaultClientOperationsLib;
using VaultClientNetLib;
using VaultLib;

namespace support
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class SupportClient
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// You can change these parameters (or read them from another source).
			string username = "admin";
			string password = "admin";
			string server = "localhost";
			string repositoryName = "Test Repository";

			// Initialize the client instance object.
			ClientInstance ci = new ClientInstance();

			// NOTE : We use the Admin access level so we can perform administrative tasks
			// NOTE : with this connection.  The username above must be an administrative
			// NOTE : user for these operations to succeed.
			ci.Init(VaultConnection.AccessLevelType.Admin);

			// Login to the server.  Use "https://" for SSL.
			ci.Login("http://" + server + "/VaultService", username, password);

			// Find the correct repository by name.
			VaultRepositoryInfo correctRepository = null;
			VaultRepositoryInfo[] reps = null;
			ci.ListRepositories(ref reps);
			foreach (VaultRepositoryInfo info in reps)
			{
				if (String.Compare(info.RepName, repositoryName, true) == 0)
				{
					correctRepository = info;
					break;
				}
			}

			if (correctRepository == null)
			{
				Console.WriteLine("Couldn't find repository '{0}'.", repositoryName);
				return;
			}

			// Set the repository we found as the locally active repository.
			ci.SetActiveRepositoryID(correctRepository.RepID, username, correctRepository.UniqueRepID, true, false);

			// Print out all the users.
			PrintUsers(ci);

			#region Add a user

			// Create a new user object to pass to the server.
			VaultUser newUser = new VaultUser();

			// For historic reasons, both of these should be set to the new user's name.
			newUser.Name = "kai";
			newUser.Login = "kai";
			
			newUser.Password = "7keys";
			newUser.Email = "kai@gamma-ray.com";
			newUser.Description = "Kai Hansen";

			// When this is set to true, the server may exceed its license limit when this user
			// is created.  You can set this to false if you're adding many users and wish to
			// activate them later (through the admin tool).
			newUser.isActive = true;

			// DefaultRights is a bitfield.  Read right is decimal 1, check out right is decimal 2, add right is decimal 4.
			newUser.DefaultRights = 1 | 2 | 4;

			// You can get the groups list from the server similar to the way you get the users list
			// above.  Just find the groups you want, and put references to them in this array and
			// the user will be created belonging to those groups.
			newUser.BelongToGroups = new VaultGroup[0];

			Console.WriteLine("Adding user {0}", newUser.Name);

			try
			{
				// We pass a ref to the object so the server can set the UserID in it.
				ci.Connection.AddUser(ref newUser);
			}
			catch (SoapException e)
			{
				Console.WriteLine("User {0} could not be added: {1}", newUser.Name, e.Message);
				ci.Logout();
				return;
			}

			Console.WriteLine("User {0} successfully added with UserID {1}", newUser.Name, newUser.UserID);

			#endregion

			// Print out all the users again, with the new one added.
			PrintUsers(ci);

			// Remember to logout.
			ci.Logout();

			// Exists so we can debug this project in Visual Studio and see any output.
			Console.WriteLine("Press Enter to finish.");
			Console.ReadLine();
		}

		private static void PrintUsers(ClientInstance ci)
		{
			VaultUser[] users = null;
			ci.ListUsers(ref users);

			// Print out the 
			Console.WriteLine("Users:");
			foreach (VaultUser user in users)
			{
				// The user item describes a user in detail.
				Console.WriteLine("\t{0} (ID: {1}, active: {2})", user.Name, user.UserID, user.isActive);
			}
		}
	}
}
